"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignalsDebugSettings = exports.SignalGlobalSettings = void 0;
const debug_mode_1 = require("../debug-mode");
const logger_1 = require("../../lib/logger");
const network_gen_1 = require("../signal-generators/network-gen");
const debug_storage_1 = require("../../lib/storage/debug-storage");
/**
 * Global settings for the application
 * In the future this pattern may allows us to add settings which can be dynamically set by the user.
 * Currently, this is just a way to pass settings to the different parts of the application.
 */
class SignalGlobalSettings {
    constructor(settings) {
        this.sampleSuccess = false;
        if (settings.maxBufferSize && settings.signalStorage) {
            throw new Error('maxBufferSize and signalStorage cannot be defined at the same time');
        }
        this.signalsDebug = new SignalsDebugSettings(settings.disableSignalsRedaction, settings.enableSignalsIngestion);
        this.signalBuffer = {
            signalStorage: settings.signalStorage,
            maxBufferSize: settings.maxBufferSize,
        };
        this.ingestClient = {
            apiHost: settings.apiHost,
            flushAt: settings.flushAt,
            flushInterval: settings.flushInterval,
            shouldDisableSignalsRedaction: this.signalsDebug.getDisableSignalsRedaction,
            shouldIngestSignals: () => {
                if (this.signalsDebug.getEnableSignalsIngestion()) {
                    return true;
                }
                if (!this.sampleSuccess) {
                    return false;
                }
                return false;
            },
        };
        this.sandbox = {
            functionHost: settings.functionHost,
            processSignal: settings.processSignal,
            edgeFnDownloadURL: undefined,
        };
        this.network = new network_gen_1.NetworkSettingsConfig({
            networkSignalsAllowList: settings.networkSignalsAllowList,
            networkSignalsDisallowList: settings.networkSignalsDisallowList,
            networkSignalsAllowSameDomain: settings.networkSignalsAllowSameDomain,
        });
    }
    update({ edgeFnDownloadURL, disallowListURLs, sampleRate, }) {
        edgeFnDownloadURL && (this.sandbox.edgeFnDownloadURL = edgeFnDownloadURL);
        this.network.networkSignalsFilterList.disallowed.addURLLike(...disallowListURLs.filter((val) => Boolean(val)));
        if (sampleRate && Math.random() <= sampleRate) {
            this.sampleSuccess = true;
        }
    }
}
exports.SignalGlobalSettings = SignalGlobalSettings;
class SignalsDebugSettings {
    constructor(disableRedaction, enableIngestion) {
        this.setAllDebugging = (boolean) => {
            this.storage.setDebugKey(SignalsDebugSettings.redactionKey, boolean);
            this.storage.setDebugKey(SignalsDebugSettings.ingestionKey, boolean);
            this.storage.setDebugKey(SignalsDebugSettings.logSignals, boolean);
        };
        this.getDisableSignalsRedaction = () => {
            return this.storage.getDebugKey(SignalsDebugSettings.redactionKey);
        };
        this.getEnableSignalsIngestion = () => {
            return this.storage.getDebugKey(SignalsDebugSettings.ingestionKey);
        };
        this.getEnableLogSignals = () => {
            return this.storage.getDebugKey(SignalsDebugSettings.logSignals);
        };
        this.storage = new debug_storage_1.DebugStorage('sessionStorage');
        if (typeof disableRedaction === 'boolean') {
            this.storage.setDebugKey(SignalsDebugSettings.redactionKey, disableRedaction);
        }
        if (typeof enableIngestion === 'boolean') {
            this.storage.setDebugKey(SignalsDebugSettings.ingestionKey, enableIngestion);
        }
        const debugModeInQs = (0, debug_mode_1.parseDebugModeQueryString)();
        logger_1.logger.debug('debugMode is set to true via query string');
        if (typeof debugModeInQs === 'boolean') {
            this.setAllDebugging(debugModeInQs);
        }
    }
}
exports.SignalsDebugSettings = SignalsDebugSettings;
SignalsDebugSettings.redactionKey = 'segment_signals_debug_redaction_disabled';
SignalsDebugSettings.ingestionKey = 'segment_signals_debug_ingestion_enabled';
SignalsDebugSettings.logSignals = 'segment_signals_log_signals_enabled';
//# sourceMappingURL=settings.js.map